import uuid
import json
import requests
import sseclient
from conva_ai.base import BaseClient
from conva_ai.response import ConvaAIResponse
from typing import AsyncGenerator


class AsyncConvaAI(BaseClient):

    async def invoke_capability(
        self,
        query: str,
        stream: bool = False,
        capability_group: str = "",
    ) -> AsyncGenerator[ConvaAIResponse, None]:
        app_context: dict = {}
        request_id = uuid.uuid4().hex
        response = requests.post(
            f"{self.host}/v1/assistants/{self.copilot_id}/text2action",
            json={
                "type": "text2action",
                "request_id": request_id,
                "assistant_id": self.copilot_id,
                "assistant_version": self.copilot_version,
                "device_id": str(uuid.getnode()),
                "input_query": query,
                "domain_name": self.domain,
                "app_context": app_context,
                "conversation_history": "{}" if not self.keep_conversation_history else self.history,
                "capability_group": capability_group,
            },
            headers={"Authorization": self.api_key, "Content-Type": "application/json"},
            stream=stream,
        )
        client = sseclient.SSEClient(response)  # type: ignore
        for event in client.events():
            event_data = event.data
            event_response = json.loads(event_data)
            rt = event_response.get("response_type", "assistant")

            if rt != "status":
                is_final = event_response.get("is_final", False)
                if stream:
                    yield ConvaAIResponse(**event_response)
                if is_final:
                    action_response = ConvaAIResponse(**event_response)
                    self.history = action_response.conversation_history
                    yield action_response



import uuid
import requests


class BaseClient:

    def __init__(self, copilot_id: str, copilot_version: str, api_key: str, host: str = "https://infer.conva.ai"):
        self.copilot_id: str = copilot_id
        self.api_key: str = api_key
        self.copilot_version: str = copilot_version
        self.host: str = host
        self.keep_conversation_history: bool = True
        self.domain: str = ""
        self.history: str = ""

    def clear_history(self):
        """
        Clears the history tracked by the client
        """
        self.history: str = ""

    def disable_history(self, use_history: bool):
        """
        Disables conversation history tracking by the client
        """
        self.keep_conversation_history = use_history



from pydantic import BaseModel
from typing import Dict, Any, List


class ConvaAIResponse(BaseModel):
    input_query: str
    message: str
    reason: str = ""
    response_language: str
    is_final: bool
    domain_name: str | None = None
    app_name: str | None = None
    category: str | None = None
    llm_key: str | None = None
    message_type: str | None = None
    app_action: str | None = None
    parameters: Dict[str, Any] = {}
    hints: List[str] = []
    suggestions: List[Dict[str, Any]] = []
    related_queries: List[str] = []
    conversation_history: str = ""
    is_error: bool = False
    is_unsupported: bool = False
    tool_name: str = ""
